/* 	Copyright (c) 1993 NeXT Computer, Inc.  All rights reserved. 
 *
 * tokensr.h - Token-ring IEEE 802.5 source routing utility functions.
 *
 *	We currently make these functions static inlines.  These should
 *	be considered for movement to a library and made public (after
 *	sanitizing API).  Note also that these functions probably should
 *	be enhanced to do hashing on route store/lookup.  
 *
 *
 * HISTORY
 * 7-Apr-93 Joel Greenblatt at NeXT 
 *	created
 *
 */

#import "tokendefs.h"

/*
 * Source routing table entry
 */
typedef	u_short	da_t;
typedef struct	{
	da_t	da[3];			/* node address of this entry */
	sroute_t ri;			/* routing information field */
} srtable_t;


/*
 * Encoded source-routing broadcast type (used as parameter to
 * source routing routines).
 */
typedef enum {
	SRB_OFF, 		/* no source-route broadcast */
	SRB_AR,			/* all-routes broadcast */
	SRB_SR,			/* single-route broadcast */
	SRB_INVALID		/* invalid entry */
} srbcast_t;

/*
 * Prototypes.
 */
static __inline__
void 	init_src_routing(srtable_t *srtable, int size, int *idx);
static __inline__
int 	check_mac_bcast(tokenHeader_t *th);
static __inline__
void 	make_sr_bcast(tokenHeader_t *th, srbcast_t type);
static __inline__
void 	get_src_route(srtable_t *srtable, int size, int *idx, u_char *da,
		tokenHeader_t *th);
static __inline__
void 	save_src_route(srtable_t *srtable, int size, int *idx, 
		tokenHeader_t *th);
static __inline__
void 	make_mac_reply(tokenHeader_t *th);
static __inline__
int 	get_ri_len(tokenHeader_t *th);
static __inline__
int 	get_8025_hdr_len(tokenHeader_t *th);
static __inline__
int 	get_unused_hdr_len(tokenHeader_t *th);


#define	NOMATCH	(-1)

/*
 * Initialize callers source routing table.
 */
static __inline__
void init_src_routing(srtable_t *srtable, int size, int *idx) {

	int i;

	*idx = 0;

	for(i=0; i < size; i++)
		srtable[i].ri.rc.len = 0; /* mark entry as "free" */

} 


/*
 * Search for a source route (given a destination address).
 */
static __inline__
int find_sr(srtable_t *srtable, int *idx, da_t *da) {

	int i;
	
	for(i=0; i < *idx; i++) {
	   	if((srtable[i].da[0] == (da[0] & ~0x8000)) &&
	      	   (srtable[i].da[1] ==  da[1]) &&	   
	      	   (srtable[i].da[2] ==  da[2])) {
		   	return(i); 
		}
	}

	return(NOMATCH);

}

/*
 * Add an entry to the callers source routing table.
 */
static __inline__
void add_sr(srtable_t *srtable, int size, int *idx,
	int i, da_t *da, sroute_t *rip) {

	if(i >= size) {
		// printf("source routing table overflow\n");}
		return;
	} 
	
	if((rip->rc.len > 18) || (rip->rc.len < 2) || (rip->rc.len & 1)) {
	      // printf("bad ri field length in rcv'd frame\n");
	} else {
		da[0] &= ~0x8000;	// no rii bit when used as a dest addr
	      	bcopy(da, &srtable[i].da, TR_DA_SIZE);
	      	bcopy(rip, &srtable[i].ri, rip->rc.len);
	      	srtable[i].ri.rc.bcast = 0;  /* make non-bcast */
	      	srtable[i].ri.rc.dir = ~srtable[i].ri.rc.dir;   
	      	srtable[i].da[0] &= ~TR_RII;	 /* mask ri bit */  
	      	if((i == *idx) && ((*idx + 1) < size)) ++*idx;
	}
	
} 

/* 
 * Find & return the source route to the callers address. 
 */
static __inline__
void get_src_route(srtable_t *srtable, int size, int *idx,
	u_char *da, tokenHeader_t *th) {

	int i;
	da_t *dest = (da_t *)da;

	if(dest[0] & 0x8000) return;	// don't handle group addresses
	 
	/* 
	 * find source route in srtable and copy to callers 
	 * tokenHeader_t (or turn off sri bit).
	 */ 
	if((i = find_sr(srtable, idx, dest)) != NOMATCH) {
	   	bcopy(&srtable[i].ri, &th->ri, srtable[i].ri.rc.len);
	   	th->sa[0] = TR_RII;
	} else {
		th->sa[0] = 0; 	/* turn off source routing indicator bit */
	}
	
} 

/* 
 * Save the source route in the callers MAC header. 
 */
static __inline__
void save_src_route(srtable_t *srtable, int size, int *idx,
	tokenHeader_t *th) {

	int i;

	/* 
	 * If frame has a routing field > 2 then save it (i.e. it's been
	 * thru at least one bridge). 
	 */
	if((th->sa[0] & TR_RII) && (th->ri.rc.len > 2)){
	   	i = find_sr(srtable, idx, (da_t *)th->sa);
	   	if(i == NOMATCH) {
	   		i = *idx;
		}
	   	add_sr(srtable, size, idx, i, (da_t *)th->sa, &th->ri);
	}	   

} 


/*
 * Returns length of the source routing field in callers MAC header.
 * Returns -1 if the header is invalid.
 */
static __inline__
int get_ri_len(tokenHeader_t *th) {

	int 		ri_len = 0;
	sroute_t 	*rif = (sroute_t *)&th->ri;

	if(th->sa[0] & 0x80) {
		ri_len = (int)rif->rc.len;
		if((ri_len & 1) || (ri_len < 2) || (ri_len > 18)) {
			// printf("bogus ri = %04x\n", th->ri.rc);
			ri_len = -1;
		}
	}
	
	return(ri_len);
}

/*
 * Returns the length of an 802.5 MAC header (including routing field).
 */
static __inline__
int get_8025_hdr_len(tokenHeader_t *th) {

	int ri_len;

	ri_len = get_ri_len(th);
	if(ri_len < 0) return(ri_len); // bad header

	return(ri_len + MAC_HDR_MIN);
}

/*
 * Returns the unused length of the callers routing field.
 * Returns -1 if error.
 */
static __inline__
int get_unused_hdr_len(tokenHeader_t *th) {

	int ri_len;

	ri_len = get_ri_len(th);
	if(ri_len < 0) return(ri_len); // bad header

	return(MAC_HDR_MAX - ri_len);
}


/*
 * Returns 1 if mac address is any type of broadcast, zero otherwise.
 */
static __inline__
int check_mac_bcast(tokenHeader_t *th) {

	if(th->da[0] & 0x80) {
		return(1); // group address (I/G bit)
	} else {
		return(0); 
	}
	
}

/*
 * Build a broadcast routing field in the callers MAC header.
 */
static __inline__
void make_sr_bcast(tokenHeader_t *th, srbcast_t type) {

	if((type == SRB_OFF) || (type >= SRB_INVALID)) {
		th->sa[0] = 0;   	
		return;
	}

	th->sa[0] = TR_RII;	/* turn on rii bit to ind. src rtng field */	 

	/*
	 * Build the routing control field for the requested
	 * broadcast type.
	 */
	if(type == SRB_AR) {
		th->ri.rc.bcast = BI_AR_BCAST;
	} else {
		th->ri.rc.bcast = BI_SR_BCAST;
	}

	th->ri.rc.len = 2;
	th->ri.rc.dir = 0;
	th->ri.rc.longf = LF_BCAST;
	th->ri.rc.rsrvd = 0;

}

/*
 * Make the callers MAC header a reply to sender.
 */
static __inline__
void make_mac_reply(tokenHeader_t *th) {

	u_char	sri;

	/*
	 * copy source address to destination address (we must preserve
	 * the routing indicator bit in the source adr).
	 */
	sri = th->sa[0];
	th->sa[0] &= ~TR_RII;
	bcopy(th->sa, th->da, sizeof(th->da));
	th->sa[0] = sri;
	
	/*
	 * convert the source routing field to a reply (flip direction
	 * bit & turn off broadcast bits). 
	 */
	if(th->sa[0] & TR_RII) {
		th->ri.rc.dir = ~th->ri.rc.dir;
		th->ri.rc.bcast = 0;
	}
}


